@@ -1,9 +1,9 @@
 package Test::Smoke::Patcher;
 use strict;
 
-# $Id: Patcher.pm 907 2005-09-10 10:46:17Z abeltje $
+# $Id$
 use vars qw( $VERSION @EXPORT );
-$VERSION = '0.009';
+$VERSION = '0.012';
 
 use base 'Exporter';
 use File::Spec;
@@ -13,7 +13,9 @@ use Test::Smoke::Util qw( get_regen_headers );
 
 @EXPORT = qw( &TRY_REGEN_HEADERS );
 
-sub TRY_REGEN_HEADERS() { 1 }
+sub MAX_FLAG_COUNT    () { 16 }
+sub ALL_FLAGS         () { (2**MAX_FLAG_COUNT) - 1 }
+sub TRY_REGEN_HEADERS () { 1 }
 
 my %CONFIG = (
     df_ddir     => File::Spec->rel2abs( cwd ),
@@ -21,13 +23,14 @@ my %CONFIG = (
     df_pfile    => undef,
     df_patchbin => 'patch',
     df_popts    => '',       # '-p1' is added in call_patch()
-    df_flags    => 1,
+    df_flags    => 0,
+    df_regen    => 1,        # regen => set/unset TRY_REGEN_HEADERS in flags
     df_oldpatch => 0,
     df_v        => 0,
 
     valid_type => { single => 1, multi => 1 },
-    single     => [qw( pfile patchbin popts flags oldpatch )],
-    multi      => [qw( pfile patchbin popts flags oldpatch )],
+    single     => [qw( pfile patchbin popts flags regen oldpatch )],
+    multi      => [qw( pfile patchbin popts flags regen oldpatch )],
 );
 
 =head1 NAME
@@ -114,6 +117,18 @@ The patch-resource can also be specified in four (4) ways.
 
 =back
 
+=head2 TRY_REGEN_HEADERS
+
+Constant: 1
+
+=head2 MAX_FLAG_COUNT 
+
+Constant: 16
+
+=head2 ALL_FLAGS
+
+Constant: 2**MAX_FLAG_COUNT) - 1
+
 =head1 METHODS
 
 =over 4
@@ -130,6 +145,7 @@ Valid keys for C<%args>:
     * pfile:    path to either the patch (single) or a textfile (multi)
     * popts:    options to pass to 'patch' (-p1)
     * patchbin: full path to the patch binary (patch)
+    * regen:    flag to set/unset the TRY_REGEN_HEADERS flag
     * v:        verbosity 0..2
 
 =cut
@@ -156,10 +172,16 @@ sub new {
         my $value = exists $args{$_} ? $args{ $_ } : $CONFIG{ "df_$_" };
         ( $_ => $value )
     } ( v => ddir => fdir => @{ $CONFIG{ $type } } );
-    $fields{pdir} = File::Spec->rel2abs( 
+    $fields{pdir} = File::Spec->rel2abs(
         defined $fields{fdir} ? $fields{fdir} : $fields{ddir}
     );
     $fields{ptype} = $type;
+    if ($fields{regen}) {
+        $fields{flags} = ($fields{flags} | TRY_REGEN_HEADERS) & ALL_FLAGS;
+    }
+    else {
+        $fields{flags} = ($fields{flags} & ~TRY_REGEN_HEADERS) & ALL_FLAGS;
+    }
 
     bless { %fields }, $class;
 }
@@ -231,23 +253,28 @@ sub perl_regen_headers {
     return 1 unless $self->{flags} & TRY_REGEN_HEADERS;
 
     my $regen_headers = get_regen_headers( $self->{pdir} );
-    if ( $regen_headers ) {
+    my $regen_perly = $self->{perly}
+        ? qq|$^X "| . File::Spec->catfile( $self->{pdir}, 'regen_perly.pl' ).
+          qq|"|
+        : "";
+    my @regens = grep $_ => ( $regen_headers, $regen_perly );
+    for my $regen ( @regens ) {
         my $cwd = cwd;
         chdir $self->{pdir} or return;
         local *RUN_REGEN;
-        if ( open RUN_REGEN, "$regen_headers |" ) {
-            $self->{v} and print "Started [$regen_headers]\n";
+        if ( open RUN_REGEN, "$regen |" ) {
+            $self->{v} and print "Started [$regen]\n";
             while ( <RUN_REGEN> ) {
                 $self->{v} and print;
             }
             close RUN_REGEN or do {
                 require Carp;
-                Carp::carp( "Error while running [$regen_headers]" );
+                Carp::carp( "Error while running [$regen]" );
                 return;
             };
         } else {
             require Carp;
-            Carp::carp( "Could not fork [$regen_headers]" );
+            Carp::carp( "Could not fork [$regen]" );
             return;
         }
         chdir $cwd;
@@ -294,7 +321,7 @@ sub patch_single {
         close PATCH;
     }
 
-    $self->{v} > 1 and print "Get patch from $self->{pfinfo}\n";
+    $self->{v} and print "Get patch from $self->{pfinfo}\n";
     $self->call_patch( \$content, @_ );
 }
 
@@ -336,13 +363,18 @@ sub patch_multi {
         close PATCHES;
     }
 
-    $self->{v} > 1 and print "Get patchinfo from $self->{pfinfo}\n";
+    $self->{v} and print "Get patchinfo from $self->{pfinfo}\n";
 
     my $ok = 1;
     foreach my $patch ( @patches ) {
         next if $patch =~ /^\s*[#]/;
         next if $patch =~ /^\s*$/;
+        if ( $patch =~ /^\s*!\s*perly$/ ) {
+            $self->{perly} = 1;
+            next;
+        }
         my( $filename, $switches, $descr ) = split /\s*;\s*/, $patch, 3;
+        $descr = $descr ? $descr . " ($filename)" : $filename;
         eval { $self->patch_single( $filename, $switches, $descr ) };
         if ( $@ ) {
             require Carp;
