@@ -1,18 +1,26 @@
-#! /usr/bin/perl -w
+#!/usr/bin/perl -w
+
+eval 'exec /usr/bin/perl -w -S $0 ${1+"$@"}'
+    if 0; # not running under some shell
 use strict;
 $|=1;
 
-# $Id: smokeperl.pl 1106 2007-09-23 05:06:10Z abeltje $
+# $Id$
 use vars qw( $VERSION );
 $VERSION = Test::Smoke->VERSION;
 
 use Cwd;
-use File::Spec;
+use File::Spec::Functions qw(:DEFAULT rel2abs);
 use File::Path;
 use File::Copy;
-use FindBin;
-use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
-use lib $FindBin::Bin;
+my $findbin;
+use File::Basename;
+BEGIN { $findbin = rel2abs(dirname $0); }
+use lib catdir($findbin, 'lib');
+use lib catdir($findbin, 'lib', 'inc');
+use lib $findbin;
+use lib catdir($findbin, 'inc');
+
 use Config;
 use Test::Smoke::Syncer;
 use Test::Smoke::Patcher;
@@ -23,11 +31,28 @@ use Test::Smoke::Util qw( get_patch calc_timeout do_pod2usage );
 
 use Getopt::Long;
 Getopt::Long::Configure( 'pass_through' );
-my %options = ( config => 'smokecurrent_config', run => 1, pfile => undef,
-                fetch => 1, patch => 1, mail => undef, archive => undef,
-                continue => 0, ccp5p_onfail => undef, killtime => undef,
-                is56x => undef, defaultenv => undef, smartsmoke => undef,
-                delay_report => undef, v => undef, cfg => undef );
+my %options = (
+    config       => 'smokecurrent_config',
+    run          => 1,
+    pfile        => undef,
+    fetch        => 1,
+    patch        => 1,
+    mail         => undef,
+    smokedb_url  => undef,
+    ua_timeout   => undef,
+    send_out     => "never",
+    send_log     => "on_fail",
+    archive      => undef,
+    continue     => 0,
+    ccp5p_onfail => undef,
+    killtime     => undef,
+    is56x        => undef,
+    defaultenv   => undef,
+    smartsmoke   => undef,
+    delay_report => undef,
+    v            => undef,
+    cfg          => undef
+);
 
 my $myusage = "Usage: $0 [-c configname]";
 GetOptions( \%options, 
@@ -36,6 +61,7 @@ GetOptions( \%options,
     'patch!',
     'ccp5p_onfail!',
     'mail!',
+    'smokedb_url=s',
     'delay_report!',
     'run!',
     'archive!',
@@ -43,6 +69,7 @@ GetOptions( \%options,
     'defaultenv!',
     'continue!',
     'smartsmoke!',
+    'patchlevel=i',
     'snapshot|s=i',
     'killtime=s',
     'pfile=s',
@@ -85,6 +112,7 @@ It can take these options
   --is56x                  This is a perl-5.6.x smoke
   --defaultenv             Run a smoke in the default environment
   --[no]smartsmoke         Don't smoke unless patchlevel changed
+  --patchlevel <plevel>    Set old patchlevel for --smartsmoke --nofetch
   --snapshot <patchlevel>  Set a new patchlevel for snapshot smokes
   --killtime (+)hh::mm     (Re)set the guard-time for this smoke
 
@@ -100,10 +128,10 @@ front-ends internally and does some sanity checking.
 
 =cut
 
-# Try cwd() first, then $FindBin::Bin
-my $config_file = File::Spec->catfile( cwd(), $options{config} );
+# Try cwd() first, then $findbin
+my $config_file = catfile( cwd(), $options{config} );
 unless ( read_config( $config_file ) ) {
-    $config_file = File::Spec->catfile( $FindBin::Bin, $options{config} );
+    $config_file = catfile( $findbin, $options{config} );
     read_config( $config_file );
 }
 defined Test::Smoke->config_error and 
@@ -115,18 +143,20 @@ defined $options{fetch} && !$options{fetch} && !defined $options{smartsmoke}
 
 # Correction for backward compatability
 !defined $options{ $_ } && !exists $conf->{ $_ } and $options{ $_ } = 1
-    for qw( run fetch patch mail archive v );
+    for qw( run fetch patch archive v );
 !defined $options{ $_ } && !exists $conf->{ $_ } and $options{ $_ } = 0
-    for qw( delay_report );
+    for qw( delay_report mail );
 
 # Make command-line options override configfile
 defined $options{ $_ } and $conf->{ $_ } = $options{ $_ }
-    for qw( is56x defaultenv continue killtime pfile cfg delay_report
-            smartsmoke run fetch patch mail ccp5p_onfail archive v );
+    for qw( is56x defaultenv continue killtime pfile cfg delay_report v run
+            smartsmoke fetch patch mail smokedb_url ccp5p_onfail archive );
 
 # Make sure the --pfile command-line override works
 $options{pfile} and $conf->{patch_type} ||= 'multi';
 
+$options{transport} or $conf->{transport_url} = undef;
+
 if ( $options{continue} ) {
     $options{v} and print "Will try to continue current smoke\n";
 } else {
@@ -136,24 +166,28 @@ if ( $options{continue} ) {
 
 my $cwd = cwd();
 chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
-call_mktest();
+call_mktest( $cwd );
+chdir $cwd;
 unless ( $conf->{delay_report} ) {
-    genrpt();
-    mailrpt();
+    sendrpt(genrpt());
 } else {
     $conf->{v} and print "Delayed creation of the report. See 'mailrpt.pl'\n";
 }
-chdir $cwd;
 archiverpt();
 
 sub synctree {
-    my $was_patchlevel = get_patch( $conf->{ddir} ) || -1;
-    my $now_patchlevel = $was_patchlevel;
+    my $now_patchlevel = get_patch( $conf->{ddir} )->[0] || -1;
+    my $was_patchlevel = $options{smartsmoke} && $options{patchlevel}
+        ? $options{patchlevel}
+        : $now_patchlevel;
     FETCHTREE: {
         unless ( $options{fetch} && $options{run} ) {
             $conf->{v} and print "Skipping synctree\n";
             last FETCHTREE;
         }
+        # TODO: It might be very useful to do a 'make distclean'
+        # *before* syncing the tree, as the sync might update MANIFEST
+        # and make distclean will not see leftovers
         if ( $options{snapshot} ) {
             if ( $conf->{sync_type} eq 'snapshot' ||
                ( $conf->{sync_type} eq 'forest'   && 
@@ -196,6 +230,7 @@ sub patchtree {
 }
 
 sub call_mktest {
+    my $cwd = shift;
     my $timeout = 0;
     if ( $Config{d_alarm} && $conf->{killtime} ) {
         $timeout = calc_timeout( $conf->{killtime} );
@@ -204,8 +239,8 @@ sub call_mktest {
     }
     $timeout and local $SIG{ALRM} = sub {
         warn "This smoke is aborted ($conf->{killtime})\n";
-        genrpt();
-        mailrpt();
+        chdir $cwd;
+        sendrpt(genrpt());
         exit(42);
     };
     $Config{d_alarm} and alarm $timeout;
@@ -217,15 +252,45 @@ sub genrpt {
     return unless $options{run};
     my $reporter = Test::Smoke::Reporter->new( $conf );
     $reporter->write_to_file;
+    return $reporter;
 }
 
-sub mailrpt {
-    unless ( $conf->{mail} && $options{run} ) {
-        $conf->{v} and print "Skipping mailrpt\n";
+sub sendrpt {
+    my $reporter = shift;
+
+    unless ( $options{run} ) {
+        $conf->{v} and print "Skipping mailrpt and transport\n";
         return;
     }
-    my $mailer = Test::Smoke::Mailer->new( $conf->{mail_type}, $conf );
-    $mailer->mail or warn "[$conf->{mail_type}] " . $mailer->error;
+
+    if ( $conf->{mail} ) {
+        my $mailer = Test::Smoke::Mailer->new( $conf->{mail_type}, $conf );
+        $mailer->mail or warn "[$conf->{mail_type}] " . $mailer->error;
+    }
+    else {
+        $conf->{v} and print "Skipping mailrpt\n";
+    }
+
+    if ($conf->{smokedb_url}) {
+        require LWP::UserAgent;
+        my $ua = LWP::UserAgent->new(
+            agent => "Test::Smoke/$Test::Smoke::VERSION",
+            (
+                defined $conf->{ua_timeout}
+                    ? (timeout => $conf->{ua_timeout})
+                    : ()
+            ),
+        );
+        $conf->{v} and print "Posting to SmokeDB ($conf->{smokedb_url})\n";
+        my $response = $ua->post(
+            $conf->{smokedb_url},
+            {json => $reporter->smokedb_data}
+        );
+        $conf->{v} and print $response->content;
+    }
+    else {
+        $conf->{v} and print "Skipping smokedb_send\n";
+    }
 }
 
 sub archiverpt {
@@ -237,17 +302,30 @@ sub archiverpt {
             die "Cannot create '$conf->{adir}': $!";
     };
 
-    my $patch_level = get_patch( $conf->{ddir} );
+    my $patch_level = get_patch( $conf->{ddir} )->[0];
     $patch_level =~ tr/ //sd;
 
-    my $archived_rpt = "rpt${patch_level}.rpt";
+    SKIP_RPT: {
+        my $archived_rpt = "rpt${patch_level}.rpt";
+        # Do not archive if it is already done
+        last SKIP_RPT
+            if -f catfile( $conf->{adir}, $archived_rpt );
 
-    # Do not archive if it is already done
-    return if -f File::Spec->catfile( $conf->{adir}, $archived_rpt );
+        copy( catfile( $conf->{ddir}, 'mktest.rpt' ),
+              catfile( $conf->{adir}, $archived_rpt ) ) or
+            die "Cannot copy to '$archived_rpt': $!";
+    }
 
-    copy( File::Spec->catfile( $conf->{ddir}, 'mktest.rpt' ),
-          File::Spec->catfile( $conf->{adir}, $archived_rpt ) ) or
-        die "Cannot copy to '$archived_rpt': $!";
+    SKIP_OUT: {
+        my $archived_out = "out${patch_level}.out";
+        # Do not archive if it is already done
+        last SKIP_OUT
+            if -f catfile( $conf->{adir}, $archived_out );
+
+        copy( catfile( $conf->{ddir}, 'mktest.out' ),
+              catfile( $conf->{adir}, $archived_out ) ) or
+            die "Cannot copy to '$archived_out': $!";
+    }
 
     SKIP_LOG: {
         my $archived_log = "log${patch_level}.log";
@@ -255,9 +333,19 @@ sub archiverpt {
         last SKIP_LOG 
             unless -f $conf->{lfile};
         copy( $conf->{lfile},
-              File::Spec->catfile( $conf->{adir}, $archived_log ) ) or
+              catfile( $conf->{adir}, $archived_log ) ) or
             die "Cannot copy to '$archived_log': $!";
     }
+
+    SKIP_JSN: {
+        my $archived_jsn = "out${patch_level}.jsn";
+        last SKIP_JSN
+            if -f catfile($conf->{adir}, $archived_jsn);
+        copy(
+            catfile($conf->{ddir}, 'mktest.jsn'),
+            catfile($conf->{adir}, $archived_jsn)
+        ) or die "Cannot copy to $archived_jsn: $!";
+    }
 }
 
 sub snapshot_name {
@@ -279,7 +367,7 @@ L<README>, L<FAQ>, L<configsmoke.pl>, L<mktest.pl>, L<mkovz.pl>
 
 =head1 REVISION
 
-$Id: smokeperl.pl 1106 2007-09-23 05:06:10Z abeltje $
+$Id$
 
 =head1 COPYRIGHT
 
